package com.sun.imageio.plugins.common;

import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import javax.imageio.spi.ImageWriterSpi;
import javax.imageio.IIOException;
import javax.imageio.ImageTypeSpecifier;
import javax.imageio.ImageWriter;
import java.awt.image.WritableRaster;
import java.awt.image.DataBuffer;
import java.awt.image.DataBufferInt;
import java.awt.image.DataBufferUShort;
import java.awt.image.DataBufferShort;
import java.awt.image.DataBufferByte;
import java.awt.Rectangle;
import java.awt.image.Raster;
import java.awt.image.ComponentColorModel;
import java.awt.color.ColorSpace;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.MultiPixelPackedSampleModel;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.ComponentSampleModel;
import java.awt.image.ColorModel;
import java.awt.image.SampleModel;

public class ImageUtil
{
    public ImageUtil() {
    }
    
    public static final ColorModel createColorModel(SampleModel sampleModel) {
        if (sampleModel == null) {
            throw new IllegalArgumentException("sampleModel == null!");
        }
        int dataType = sampleModel.getDataType();
        switch (dataType) {
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5: {
                Object o = null;
                int[] sampleSize = sampleModel.getSampleSize();
                if ((sampleModel instanceof ComponentSampleModel)) {
                    int numBands = sampleModel.getNumBands();
                    ColorSpace colorSpace;
                    if (numBands <= 2) {
                        colorSpace = ColorSpace.getInstance(1003);
                    }
                    else if (numBands <= 4) {
                        colorSpace = ColorSpace.getInstance(1000);
                    }
                    else {
                        colorSpace = new BogusColorSpace(numBands);
                    }
                    int hasAlpha = numBands == 2 || numBands == 4;
                    o = new ComponentColorModel(colorSpace, sampleSize, (boolean)hasAlpha, false, hasAlpha ? 3 : 1, dataType);
                }
                else if (sampleModel.getNumBands() <= 4 && sampleModel instanceof SinglePixelPackedSampleModel) {
                    int[] bitMasks = ((SinglePixelPackedSampleModel)sampleModel).getBitMasks();
                    int amask = 0;
                    int length = bitMasks.length;
                    int rmask;
                    int gmask;
                    int bmask;
                    if (length <= 2) {
                        gmask = (rmask = (bmask = bitMasks[0]));
                        if (length == 2) {
                            amask = bitMasks[1];
                        }
                    }
                    else {
                        rmask = bitMasks[0];
                        gmask = bitMasks[1];
                        bmask = bitMasks[2];
                        if (length == 4) {
                            amask = bitMasks[3];
                        }
                    }
                    int bits2 = 0;
                    for (int j = 0; j < sampleSize.length; ++j) {
                        bits2 += sampleSize[j];
                    }
                    return new DirectColorModel(bits2, rmask, gmask, bmask, amask);
                }
                else if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
                    int bits = sampleSize[0];
                    int size = 1 << bits;
                    byte[] array = new byte[size];
                    for (int i = 0; i < size; ++i) {
                        array[i] = (byte)(i * 255 / (size - 1));
                    }
                    o = new IndexColorModel(bits, size, array, array, array);
                }
                return o;
            }
            default: {
                return null;
            }
        }
    }
    
    public static byte[] getPackedBinaryData(Raster raster, Rectangle rectangle) {
        SampleModel sampleModel = raster.getSampleModel();
        if (!isBinary(sampleModel)) {
            throw new IllegalArgumentException(I18N.getString("ImageUtil0"));
        }
        int x = rectangle.x;
        int y = rectangle.y;
        int i = rectangle.width;
        int height = rectangle.height;
        DataBuffer dataBuffer = raster.getDataBuffer();
        int n = x - raster.getSampleModelTranslateX();
        int y2 = y - raster.getSampleModelTranslateY();
        MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
        int scanlineStride = multiPixelPackedSampleModel.getScanlineStride();
        int n2 = dataBuffer.getOffset() + multiPixelPackedSampleModel.getOffset(n, y2);
        int bitOffset = multiPixelPackedSampleModel.getBitOffset(n);
        int n3 = (i + 7) / 8;
        if (dataBuffer instanceof DataBufferByte && n2 == 0 && bitOffset == 0 && n3 == scanlineStride && ((DataBufferByte)dataBuffer).getData().length == n3 * height) {
            return ((DataBufferByte)dataBuffer).getData();
        }
        byte[] array = new byte[n3 * height];
        int n4 = 0;
        if (bitOffset == 0) {
            if ((dataBuffer instanceof DataBufferByte)) {
                byte[] data4 = ((DataBufferByte)dataBuffer).getData();
                for (int n24 = 0, n25 = 0; n25 < height; n24 += n3, n2 += scanlineStride, ++n25) {
                    System.arraycopy(data4, n2, array, n24, n3);
                }
            }
            else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
                short[] array3 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
                for (int n22 = 0; n22 < height; n2 += scanlineStride, ++n22) {
                    while (i > 8) {
                        int n23 = array3[n2++];
                        array[n4++] = (byte)(n23 >>> 8 & 0xFF);
                        array[n4++] = (byte)(n23 & 0xFF);
                        i -= 16;
                    }
                    if (i > 0) {
                        array[n4++] = (byte)(array3[n2] >>> 8 & 0xFF);
                    }
                }
            }
            else if ((dataBuffer instanceof DataBufferInt)) {
                int[] data3 = ((DataBufferInt)dataBuffer).getData();
                for (int n19 = 0; n19 < height; n2 += scanlineStride, ++n19) {
                    while (i > 24) {
                        int n20 = data3[n2++];
                        array[n4++] = (byte)(n20 >>> 24 & 0xFF);
                        array[n4++] = (byte)(n20 >>> 16 & 0xFF);
                        array[n4++] = (byte)(n20 >>> 8 & 0xFF);
                        array[n4++] = (byte)(n20 & 0xFF);
                        i -= 32;
                    }
                    int n21 = 24;
                    while (i > 0) {
                        array[n4++] = (byte)(data3[n2] >>> n21 & 0xFF);
                        n21 -= 8;
                        i -= 8;
                    }
                }
            }
        }
        else if ((dataBuffer instanceof DataBufferByte)) {
            byte[] data2 = ((DataBufferByte)dataBuffer).getData();
            if ((bitOffset & 0x7) == 0x0) {
                for (int n17 = 0, n18 = 0; n18 < height; n17 += n3, n2 += scanlineStride, ++n18) {
                    System.arraycopy(data2, n2, array, n17, n3);
                }
            }
            else {
                int n14 = bitOffset & 0x7;
                int n15 = 8 - n14;
                for (int n16 = 0; n16 < height; n2 += scanlineStride, ++n16) {
                    for (; i > 0; i -= 8) {
                        if (i > n15) {
                            array[n4++] = (byte)((data2[n2++] & 0xFF) << n14 | (data2[n2] & 0xFF) >>> n15);
                        }
                        else {
                            array[n4++] = (byte)((data2[n2] & 0xFF) << n14);
                        }
                    }
                }
            }
        }
        else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
            short[] array2 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
            for (int l = 0; l < height; n2 += scanlineStride, ++l) {
                for (int n9 = 0; n9 < i; n9 += 8, bitOffset += 8) {
                    int n10 = n2 + bitOffset / 16;
                    int n11 = bitOffset % 16;
                    int n12 = array2[n10] & 0xFFFF;
                    if (n11 <= 8) {
                        array[n4++] = (byte)(n12 >>> 8 - n11);
                    }
                    else {
                        int n13 = n11 - 8;
                        array[n4++] = (byte)(n12 << n13 | (array2[n10 + 1] & 0xFFFF) >>> 16 - n13);
                    }
                }
            }
        }
        else if ((dataBuffer instanceof DataBufferInt)) {
            int[] data = ((DataBufferInt)dataBuffer).getData();
            for (int j = 0; j < height; n2 += scanlineStride, ++j) {
                for (int k = 0; k < i; k += 8, bitOffset += 8) {
                    int n5 = n2 + bitOffset / 32;
                    int n6 = bitOffset % 32;
                    int n7 = data[n5];
                    if (n6 <= 24) {
                        array[n4++] = (byte)(n7 >>> 24 - n6);
                    }
                    else {
                        int n8 = n6 - 24;
                        array[n4++] = (byte)(n7 << n8 | data[n5 + 1] >>> 32 - n8);
                    }
                }
            }
        }
        return array;
    }
    
    public static byte[] getUnpackedBinaryData(Raster raster, Rectangle rectangle) {
        SampleModel sampleModel = raster.getSampleModel();
        if (!isBinary(sampleModel)) {
            throw new IllegalArgumentException(I18N.getString("ImageUtil0"));
        }
        int i = rectangle.x;
        int j = rectangle.y;
        int width = rectangle.width;
        int height = rectangle.height;
        DataBuffer dataBuffer = raster.getDataBuffer();
        int n = i - raster.getSampleModelTranslateX();
        int y = j - raster.getSampleModelTranslateY();
        MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
        int scanlineStride = multiPixelPackedSampleModel.getScanlineStride();
        int n2 = dataBuffer.getOffset() + multiPixelPackedSampleModel.getOffset(n, y);
        int bitOffset = multiPixelPackedSampleModel.getBitOffset(n);
        byte[] array = new byte[width * height];
        int n3 = j + height;
        int n4 = i + width;
        int n5 = 0;
        if ((dataBuffer instanceof DataBufferByte)) {
            byte[] data2 = ((DataBufferByte)dataBuffer).getData();
            for (; j < n3; n2 += scanlineStride, ++j) {
                int n8 = n2 * 8 + bitOffset;
                for (; i < n4; ++i) {
                    array[n5++] = (byte)(data2[n8 / 8] >>> (7 - n8 & 0x7) & 0x1);
                    ++n8;
                }
            }
        }
        else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
            short[] array2 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
            for (; j < n3; n2 += scanlineStride, ++j) {
                int n7 = n2 * 16 + bitOffset;
                for (; i < n4; ++i) {
                    array[n5++] = (byte)(array2[n7 / 16] >>> 15 - n7 % 16 & 0x1);
                    ++n7;
                }
            }
        }
        else if ((dataBuffer instanceof DataBufferInt)) {
            int[] data = ((DataBufferInt)dataBuffer).getData();
            for (; j < n3; n2 += scanlineStride, ++j) {
                int n6 = n2 * 32 + bitOffset;
                for (; i < n4; ++i) {
                    array[n5++] = (byte)(data[n6 / 32] >>> 31 - n6 % 32 & 0x1);
                    ++n6;
                }
            }
        }
        return array;
    }
    
    public static void setPackedBinaryData(byte[] array, WritableRaster writableRaster, Rectangle rectangle) {
        SampleModel sampleModel = writableRaster.getSampleModel();
        if (!isBinary(sampleModel)) {
            throw new IllegalArgumentException(I18N.getString("ImageUtil0"));
        }
        int x = rectangle.x;
        int y = rectangle.y;
        int i = rectangle.width;
        int height = rectangle.height;
        DataBuffer dataBuffer = writableRaster.getDataBuffer();
        int n = x - writableRaster.getSampleModelTranslateX();
        int y2 = y - writableRaster.getSampleModelTranslateY();
        MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
        int scanlineStride = multiPixelPackedSampleModel.getScanlineStride();
        int n2 = dataBuffer.getOffset() + multiPixelPackedSampleModel.getOffset(n, y2);
        int bitOffset = multiPixelPackedSampleModel.getBitOffset(n);
        int n3 = 0;
        if (bitOffset == 0) {
            if ((dataBuffer instanceof DataBufferByte)) {
                byte[] data4 = ((DataBufferByte)dataBuffer).getData();
                if (data4 == array) {
                    return;
                }
                for (int n40 = (i + 7) / 8, n41 = 0, n42 = 0; n42 < height; n41 += n40, n2 += scanlineStride, ++n42) {
                    System.arraycopy(array, n41, data4, n2, n40);
                }
            }
            else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
                short[] array3 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
                for (int n39 = 0; n39 < height; n2 += scanlineStride, ++n39) {
                    while (i > 8) {
                        array3[n2++] = (short)((array[n3++] & 0xFF) << 8 | (array[n3++] & 0xFF));
                        i -= 16;
                    }
                    if (i > 0) {
                        array3[n2++] = (short)((array[n3++] & 0xFF) << 8);
                    }
                }
            }
            else if ((dataBuffer instanceof DataBufferInt)) {
                int[] data3 = ((DataBufferInt)dataBuffer).getData();
                for (int n36 = 0; n36 < height; n2 += scanlineStride, ++n36) {
                    while (i > 24) {
                        data3[n2++] = ((array[n3++] & 0xFF) << 24 | (array[n3++] & 0xFF) << 16 | (array[n3++] & 0xFF) << 8 | (array[n3++] & 0xFF));
                        i -= 32;
                    }
                    int n37 = 24;
                    while (i > 0) {
                        int n38 = n2;
                        data3[n38] |= (array[n3++] & 0xFF) << n37;
                        n37 -= 8;
                        i -= 8;
                    }
                }
            }
        }
        else {
            int n4 = (i + 7) / 8;
            int n5 = 0;
            if ((dataBuffer instanceof DataBufferByte)) {
                byte[] data2 = ((DataBufferByte)dataBuffer).getData();
                if ((bitOffset & 0x7) == 0x0) {
                    for (int n35 = 0; n35 < height; n5 += n4, n2 += scanlineStride, ++n35) {
                        System.arraycopy(array, n5, data2, n2, n4);
                    }
                }
                else {
                    int n27 = bitOffset & 0x7;
                    int n28 = 8 - n27;
                    int n29 = 8 + n28;
                    int n30 = (byte)(255 << n28);
                    int n31 = (byte)~n30;
                    for (int n32 = 0; n32 < height; n2 += scanlineStride, ++n32) {
                        for (; i > 0; i -= 8) {
                            int n33 = array[n3++];
                            if (i > n29) {
                                data2[n2] = (byte)((data2[n2] & n30) | (n33 & 0xFF) >>> n27);
                                data2[++n2] = (byte)((n33 & 0xFF) << n28);
                            }
                            else if (i > n28) {
                                data2[n2] = (byte)((data2[n2] & n30) | (n33 & 0xFF) >>> n27);
                                ++n2;
                                data2[n2] = (byte)((data2[n2] & n31) | (n33 & 0xFF) << n28);
                            }
                            else {
                                int n34 = (1 << n28 - i) - 1;
                                data2[n2] = (byte)((data2[n2] & (n30 | n34)) | ((n33 & 0xFF) >>> n27 & ~n34));
                            }
                        }
                    }
                }
            }
            else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
                short[] array2 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
                int n16 = bitOffset & 0x7;
                int n17 = 8 - n16;
                int n18 = 16 + n17;
                int n19 = (short)~(255 << n17);
                int n20 = (short)(65535 << n17);
                int n21 = (short)~n20;
                for (int l = 0; l < height; n2 += scanlineStride, ++l) {
                    for (int n22 = 0; n22 < i; n22 += 8, bitOffset += 8, i -= 8) {
                        int n23 = n2 + (bitOffset >> 4);
                        int n24 = bitOffset & 0xF;
                        int n25 = array[n3++] & 0xFF;
                        if (n24 <= 8) {
                            if (i < 8) {
                                n25 &= 255 << 8 - i;
                            }
                            array2[n23] = (short)((array2[n23] & n19) | n25 << n17);
                        }
                        else if (i > n18) {
                            array2[n23] = (short)((array2[n23] & n20) | (n25 >>> n16 & 0xFFFF));
                            array2[++n23] = (short)(n25 << n17 & 0xFFFF);
                        }
                        else if (i > n17) {
                            array2[n23] = (short)((array2[n23] & n20) | (n25 >>> n16 & 0xFFFF));
                            ++n23;
                            array2[n23] = (short)((array2[n23] & n21) | (n25 << n17 & 0xFFFF));
                        }
                        else {
                            int n26 = (1 << n17 - i) - 1;
                            array2[n23] = (short)((array2[n23] & (n20 | n26)) | (n25 >>> n16 & 0xFFFF & ~n26));
                        }
                    }
                }
            }
            else if ((dataBuffer instanceof DataBufferInt)) {
                int[] data = ((DataBufferInt)dataBuffer).getData();
                int n6 = bitOffset & 0x7;
                int n7 = 8 - n6;
                int n8 = 32 + n7;
                int n9 = -1 << n7;
                int n10 = ~n9;
                for (int j = 0; j < height; n2 += scanlineStride, ++j) {
                    for (int k = 0; k < i; k += 8, bitOffset += 8, i -= 8) {
                        int n11 = n2 + (bitOffset >> 5);
                        int n12 = bitOffset & 0x1F;
                        int n13 = array[n3++] & 0xFF;
                        if (n12 <= 24) {
                            int n15 = 24 - n12;
                            if (i < 8) {
                                n13 &= 255 << 8 - i;
                            }
                            data[n11] = ((data[n11] & ~(255 << n15)) | n13 << n15);
                        }
                        else if (i > n8) {
                            data[n11] = ((data[n11] & n9) | n13 >>> n6);
                            data[++n11] = n13 << n7;
                        }
                        else if (i > n7) {
                            data[n11] = ((data[n11] & n9) | n13 >>> n6);
                            ++n11;
                            data[n11] = ((data[n11] & n10) | n13 << n7);
                        }
                        else {
                            int n14 = (1 << n7 - i) - 1;
                            data[n11] = ((data[n11] & (n9 | n14)) | (n13 >>> n6 & ~n14));
                        }
                    }
                }
            }
        }
        return;
    }
    
    public static void setUnpackedBinaryData(byte[] array, WritableRaster writableRaster, Rectangle rectangle) {
        SampleModel sampleModel = writableRaster.getSampleModel();
        if (!isBinary(sampleModel)) {
            throw new IllegalArgumentException(I18N.getString("ImageUtil0"));
        }
        int x = rectangle.x;
        int y = rectangle.y;
        int width = rectangle.width;
        int height = rectangle.height;
        DataBuffer dataBuffer = writableRaster.getDataBuffer();
        int n = x - writableRaster.getSampleModelTranslateX();
        int y2 = y - writableRaster.getSampleModelTranslateY();
        MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
        int scanlineStride = multiPixelPackedSampleModel.getScanlineStride();
        int n2 = dataBuffer.getOffset() + multiPixelPackedSampleModel.getOffset(n, y2);
        int bitOffset = multiPixelPackedSampleModel.getBitOffset(n);
        int n3 = 0;
        if ((dataBuffer instanceof DataBufferByte)) {
            byte[] data2 = ((DataBufferByte)dataBuffer).getData();
            for (int n8 = 0; n8 < height; n2 += scanlineStride, ++n8) {
                for (int n9 = n2 * 8 + bitOffset, n10 = 0; n10 < width; ++n9, ++n10) {
                    if (array[n3++] != 0) {
                        int n11 = n9 / 8;
                        data2[n11] |= (byte)(1 << (7 - n9 & 0x7));
                    }
                }
            }
        }
        else if (dataBuffer instanceof DataBufferShort || dataBuffer instanceof DataBufferUShort) {
            short[] array2 = (dataBuffer instanceof DataBufferShort) ? ((DataBufferShort)dataBuffer).getData() : ((DataBufferUShort)dataBuffer).getData();
            for (int k = 0; k < height; n2 += scanlineStride, ++k) {
                for (int n6 = n2 * 16 + bitOffset, l = 0; l < width; ++n6, ++l) {
                    if (array[n3++] != 0) {
                        int n7 = n6 / 16;
                        array2[n7] |= (short)(1 << 15 - n6 % 16);
                    }
                }
            }
        }
        else if ((dataBuffer instanceof DataBufferInt)) {
            int[] data = ((DataBufferInt)dataBuffer).getData();
            for (int i = 0; i < height; n2 += scanlineStride, ++i) {
                for (int n4 = n2 * 32 + bitOffset, j = 0; j < width; ++n4, ++j) {
                    if (array[n3++] != 0) {
                        int n5 = n4 / 32;
                        data[n5] |= 1 << 31 - n4 % 32;
                    }
                }
            }
        }
        return;
    }
    
    public static boolean isBinary(SampleModel sampleModel) {
        return sampleModel instanceof MultiPixelPackedSampleModel && ((MultiPixelPackedSampleModel)sampleModel).getPixelBitStride() == 1 && sampleModel.getNumBands() == 1;
    }
    
    public static ColorModel createColorModel(ColorSpace instance, SampleModel sampleModel) {
        Object o = null;
        if (sampleModel == null) {
            throw new IllegalArgumentException(I18N.getString("ImageUtil1"));
        }
        int numBands = sampleModel.getNumBands();
        if (numBands < 1 || numBands > 4) {
            return null;
        }
        int dataType = sampleModel.getDataType();
        if ((sampleModel instanceof ComponentSampleModel)) {
            if (dataType < 0 || dataType > 5) {
                return null;
            }
            if (instance == null) {
                instance = ((numBands <= 2) ? ColorSpace.getInstance(1003) : ColorSpace.getInstance(1000));
            }
            int hasAlpha = numBands == 2 || numBands == 4;
            int transparency = hasAlpha ? 3 : 1;
            boolean isAlphaPremultiplied = false;
            int dataTypeSize = DataBuffer.getDataTypeSize(dataType);
            int[] bits2 = new int[numBands];
            for (int k = 0; k < numBands; ++k) {
                bits2[k] = dataTypeSize;
            }
            o = new ComponentColorModel(instance, bits2, (boolean)hasAlpha, (boolean)isAlphaPremultiplied, transparency, dataType);
        }
        else if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)sampleModel;
            int[] bitMasks = singlePixelPackedSampleModel.getBitMasks();
            int amask = 0;
            int length = bitMasks.length;
            int rmask;
            int gmask;
            int bmask;
            if (length <= 2) {
                gmask = (rmask = (bmask = bitMasks[0]));
                if (length == 2) {
                    amask = bitMasks[1];
                }
            }
            else {
                rmask = bitMasks[0];
                gmask = bitMasks[1];
                bmask = bitMasks[2];
                if (length == 4) {
                    amask = bitMasks[3];
                }
            }
            int[] sampleSize = singlePixelPackedSampleModel.getSampleSize();
            int bits = 0;
            for (int j = 0; j < sampleSize.length; ++j) {
                bits += sampleSize[j];
            }
            if (instance == null) {
                instance = ColorSpace.getInstance(1000);
            }
            o = new DirectColorModel(instance, bits, rmask, gmask, bmask, amask, false, sampleModel.getDataType());
        }
        else if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
            int pixelBitStride = ((MultiPixelPackedSampleModel)sampleModel).getPixelBitStride();
            int size = 1 << pixelBitStride;
            byte[] array = new byte[size];
            for (int i = 0; i < size; ++i) {
                array[i] = (byte)(255 * i / (size - 1));
            }
            o = new IndexColorModel(pixelBitStride, size, array, array, array);
        }
        return o;
    }
    
    public static int getElementSize(SampleModel sampleModel) {
        int dataTypeSize = DataBuffer.getDataTypeSize(sampleModel.getDataType());
        if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
            MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
            return multiPixelPackedSampleModel.getSampleSize(0) * multiPixelPackedSampleModel.getNumBands();
        }
        if ((sampleModel instanceof ComponentSampleModel)) {
            return sampleModel.getNumBands() * dataTypeSize;
        }
        if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
            return dataTypeSize;
        }
        return dataTypeSize * sampleModel.getNumBands();
    }
    
    public static long getTileSize(SampleModel sampleModel) {
        int dataTypeSize = DataBuffer.getDataTypeSize(sampleModel.getDataType());
        if ((sampleModel instanceof MultiPixelPackedSampleModel)) {
            MultiPixelPackedSampleModel multiPixelPackedSampleModel = (MultiPixelPackedSampleModel)sampleModel;
            return (long)((multiPixelPackedSampleModel.getScanlineStride() * multiPixelPackedSampleModel.getHeight() + (multiPixelPackedSampleModel.getDataBitOffset() + dataTypeSize - 1) / dataTypeSize) * ((dataTypeSize + 7) / 8));
        }
        if ((sampleModel instanceof ComponentSampleModel)) {
            ComponentSampleModel componentSampleModel = (ComponentSampleModel)sampleModel;
            int[] bandOffsets = componentSampleModel.getBandOffsets();
            int max = bandOffsets[0];
            for (int i = 1; i < bandOffsets.length; ++i) {
                max = Math.max(max, bandOffsets[i]);
            }
            long n = 0L;
            int pixelStride = componentSampleModel.getPixelStride();
            int scanlineStride = componentSampleModel.getScanlineStride();
            if (max >= 0) {
                n += (long)(max + 1);
            }
            if (pixelStride > 0) {
                n += (long)(pixelStride * (sampleModel.getWidth() - 1));
            }
            if (scanlineStride > 0) {
                n += (long)(scanlineStride * (sampleModel.getHeight() - 1));
            }
            int[] bankIndices = componentSampleModel.getBankIndices();
            int max2 = bankIndices[0];
            for (int j = 1; j < bankIndices.length; ++j) {
                max2 = Math.max(max2, bankIndices[j]);
            }
            return n * (long)(max2 + 1) * (long)((dataTypeSize + 7) / 8);
        }
        else {
            if ((sampleModel instanceof SinglePixelPackedSampleModel)) {
                SinglePixelPackedSampleModel singlePixelPackedSampleModel = (SinglePixelPackedSampleModel)sampleModel;
                return (long)(singlePixelPackedSampleModel.getScanlineStride() * (singlePixelPackedSampleModel.getHeight() - 1) + singlePixelPackedSampleModel.getWidth()) * (long)((dataTypeSize + 7) / 8);
            }
            return 0L;
        }
    }
    
    public static long getBandSize(SampleModel sampleModel) {
        int dataTypeSize = DataBuffer.getDataTypeSize(sampleModel.getDataType());
        if ((sampleModel instanceof ComponentSampleModel)) {
            ComponentSampleModel componentSampleModel = (ComponentSampleModel)sampleModel;
            int pixelStride = componentSampleModel.getPixelStride();
            int scanlineStride = componentSampleModel.getScanlineStride();
            long n = (long)Math.min(pixelStride, scanlineStride);
            if (pixelStride > 0) {
                n += (long)(pixelStride * (sampleModel.getWidth() - 1));
            }
            if (scanlineStride > 0) {
                n += (long)(scanlineStride * (sampleModel.getHeight() - 1));
            }
            return n * (long)((dataTypeSize + 7) / 8);
        }
        else {
            return getTileSize(sampleModel);
        }
    }
    
    public static boolean isIndicesForGrayscale(byte[] array, byte[] array2, byte[] array3) {
        if (array.length != array2.length || array.length != array3.length) {
            return false;
        }
        int length = array.length;
        if (length != 256) {
            return false;
        }
        int i = 0;
        while (i < length) {
            int n = (byte)i;
            if (array[i] != n || array2[i] != n || array3[i] != n) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    public static String convertObjectToString(Object o) {
        if (o == null) {
            return "";
        }
        String str = "";
        if ((o instanceof byte[])) {
            byte[] array3 = (byte[])(byte[])o;
            for (int k = 0; k < array3.length; ++k) {
                str = str + (int)array3[k] + " ";
            }
            return str;
        }
        if ((o instanceof int[])) {
            int[] array2 = (int[])(int[])o;
            for (int j = 0; j < array2.length; ++j) {
                str = str + array2[j] + " ";
            }
            return str;
        }
        if ((o instanceof short[])) {
            short[] array = (short[])(short[])o;
            for (int i = 0; i < array.length; ++i) {
                str = str + (int)array[i] + " ";
            }
            return str;
        }
        return o.toString();
    }
    
    public static final void canEncodeImage(ImageWriter imageWriter, ImageTypeSpecifier imageTypeSpecifier) throws IIOException {
        ImageWriterSpi originatingProvider = imageWriter.getOriginatingProvider();
        if (imageTypeSpecifier != null && originatingProvider != null && !originatingProvider.canEncodeImage(imageTypeSpecifier)) {
            throw new IIOException(I18N.getString("ImageUtil2") + " " + imageWriter.getClass().getName());
        }
        return;
    }
    
    public static final void canEncodeImage(ImageWriter imageWriter, ColorModel colorModel, SampleModel sampleModel) throws IIOException {
        ImageTypeSpecifier imageTypeSpecifier = null;
        if (colorModel != null && sampleModel != null) {
            imageTypeSpecifier = new ImageTypeSpecifier(colorModel, sampleModel);
        }
        canEncodeImage(imageWriter, imageTypeSpecifier);
    }
    
    public static final boolean imageIsContiguous(RenderedImage renderedImage) {
        SampleModel sampleModel;
        if ((renderedImage instanceof BufferedImage)) {
            sampleModel = ((BufferedImage)renderedImage).getRaster().getSampleModel();
        }
        else {
            sampleModel = renderedImage.getSampleModel();
        }
        if ((sampleModel instanceof ComponentSampleModel)) {
            ComponentSampleModel componentSampleModel = (ComponentSampleModel)sampleModel;
            if (componentSampleModel.getPixelStride() != componentSampleModel.getNumBands()) {
                return false;
            }
            int[] bandOffsets = componentSampleModel.getBandOffsets();
            int i = 0;
            while (i < bandOffsets.length) {
                if (bandOffsets[i] != i) {
                    return false;
                }
                ++i;
            }
            int[] bankIndices = componentSampleModel.getBankIndices();
            int j = 0;
            while (j < bandOffsets.length) {
                if (bankIndices[j] != 0) {
                    return false;
                }
                ++j;
            }
            return true;
        }
        else {
            return isBinary(sampleModel);
        }
    }
}
